@@ -40,11 +40,9 @@ typedef struct {
   GtkMenu *menu;
 
   struct {
-    gboolean(*func)(GtkTreeSelection *, GtkTreeModel *,
-		    GtkTreePath *, gboolean,
-		    gpointer);
+    void(*func)(GtkTreeSelection *, gpointer);
     gpointer data;
-  } sel;
+  } change;
 
   GtkWidget *table;
 
@@ -56,49 +54,6 @@ typedef struct {
 
 } list_priv_t;
 
-#if defined(USE_HILDON) && (MAEMO_VERSION_MAJOR == 5)
-#define FREMANTLE_PANNABLE_AREA
-#include <hildon/hildon-gtk.h>
-#include <hildon/hildon-pannable-area.h>
-// #define FREMANTLE_USE_POPUP
-#endif
-
-#ifdef FREMANTLE_USE_POPUP
-
-static void cmenu_init(GtkWidget *list) {
-  list_priv_t *priv = g_object_get_data(G_OBJECT(list), "priv");
-  g_assert(priv);
-
-  /* Create needed handles. */
-  priv->menu = GTK_MENU(gtk_menu_new());
-
-  gtk_widget_tap_and_hold_setup(priv->view, GTK_WIDGET(priv->menu), NULL, 0);
-}
-
-static GtkWidget *cmenu_append(GtkWidget *list, char *label, 
-			       GCallback cb, gpointer data) {
-  list_priv_t *priv = g_object_get_data(G_OBJECT(list), "priv");
-  g_assert(priv);
-
-  GtkWidget *menu_item;
-
-  /* Setup the map context menu. */
-  gtk_menu_append(priv->menu, menu_item
-		  = gtk_menu_item_new_with_label(label));
-
-  hildon_gtk_widget_set_theme_size(menu_item, 
-	   (HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH));
-
-  gtk_signal_connect(GTK_OBJECT(menu_item), "activate", 
-		     GTK_SIGNAL_FUNC(cb), data);
-
-  gtk_widget_show_all(GTK_WIDGET(priv->menu));
-
-  return menu_item;
-}
-
-#endif
-
 GtkWidget *list_get_view(GtkWidget *list) {
   list_priv_t *priv = g_object_get_data(G_OBJECT(list), "priv");
   g_assert(priv);
@@ -114,7 +69,10 @@ void list_set_user_buttons(GtkWidget *list, ...) {
   va_list ap;
 
   /* make space for user buttons */
-  gtk_table_resize(GTK_TABLE(priv->table), 2, 3);
+  if(!(priv->button.flags & LIST_BTN_WIDE))
+    gtk_table_resize(GTK_TABLE(priv->table), 2, 3);
+  else
+    gtk_table_resize(GTK_TABLE(priv->table), 1, 5);
 
   va_start(ap, list);
   list_button_t id = va_arg(ap, list_button_t);
@@ -122,14 +80,14 @@ void list_set_user_buttons(GtkWidget *list, ...) {
     char *label = va_arg(ap, char*);
     GCallback cb = va_arg(ap, GCallback);
 
-    priv->button.widget[id] = gtk_button_new_with_label(label);
-#ifdef FREMANTLE_PANNABLE_AREA
-    if(priv->button.flags & LIST_BTN_BIG)
-      hildon_gtk_widget_set_theme_size(priv->button.widget[id], 
-	       (HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH));
-#endif
-    gtk_table_attach_defaults(GTK_TABLE(priv->table), priv->button.widget[id], 
-	      id-LIST_BUTTON_USER0, id-LIST_BUTTON_USER0+1, 1, 2);
+    priv->button.widget[id] = button_new_with_label(label);
+    if(!(priv->button.flags & LIST_BTN_WIDE))
+      gtk_table_attach_defaults(GTK_TABLE(priv->table), priv->button.widget[id], 
+		id-LIST_BUTTON_USER0, id-LIST_BUTTON_USER0+1, 1, 2);
+    else
+      gtk_table_attach_defaults(GTK_TABLE(priv->table), priv->button.widget[id], 
+		3+id-LIST_BUTTON_USER0, 3+id-LIST_BUTTON_USER0+1, 0, 1);
+
     gtk_signal_connect(GTK_OBJECT(priv->button.widget[id]), "clicked", 
 		       GTK_SIGNAL_FUNC(cb), priv->button.data);
 
@@ -218,8 +176,13 @@ void list_set_custom_user_button(GtkWidget *list, list_button_t id,
   /* make space for user buttons */
   gtk_table_resize(GTK_TABLE(priv->table), 2, 3);
   
-  gtk_table_attach_defaults(GTK_TABLE(priv->table), widget,
-		    id-LIST_BUTTON_USER0, id-LIST_BUTTON_USER0+1, 1, 2);
+  if(!(priv->button.flags & LIST_BTN_WIDE))
+    gtk_table_attach_defaults(GTK_TABLE(priv->table), widget,
+	      id-LIST_BUTTON_USER0, id-LIST_BUTTON_USER0+1, 1, 2);
+  else
+    gtk_table_attach_defaults(GTK_TABLE(priv->table), widget,
+	      3+id-LIST_BUTTON_USER0, 3+id-LIST_BUTTON_USER0+1, 0, 1);
+
   priv->button.widget[id] = widget;
 }
 
@@ -233,15 +196,32 @@ GtkTreeSelection *list_get_selection(GtkWidget *list) {
   return sel;
 }
 
+/* returns true if something is selected. on in mode multiple returns */
+/* true if exactly one item is selected */
 gboolean list_get_selected(GtkWidget *list, GtkTreeModel **model,
 			   GtkTreeIter *iter) {
+  gboolean retval = FALSE;
   list_priv_t *priv = g_object_get_data(G_OBJECT(list), "priv");
   g_assert(priv);
 
+#if 1  
+  // this copes with multiple selections ...
   GtkTreeSelection *sel =
     gtk_tree_view_get_selection(GTK_TREE_VIEW(priv->view));
 
+  GList *slist = gtk_tree_selection_get_selected_rows(sel, model);
+  
+  if(g_list_length(slist) == 1) 
+    retval = gtk_tree_model_get_iter(*model, iter, slist->data);
+
+  g_list_foreach(slist, (GFunc)gtk_tree_path_free, NULL);
+  g_list_free(slist);
+
+  return retval;
+#else
+  // ... this doesn't
   return gtk_tree_selection_get_selected(sel, model, iter);
+#endif
 }
 
 void list_button_enable(GtkWidget *list, list_button_t id, gboolean enable) {
@@ -256,41 +236,6 @@ void list_set_store(GtkWidget *list, GtkListStore *store) {
   gtk_tree_view_set_model(GTK_TREE_VIEW(priv->view), GTK_TREE_MODEL(store));
 }
 
-void list_set_selection_function(GtkWidget *list, GtkTreeSelectionFunc func,
-				 gpointer data) {
-  list_priv_t *priv = g_object_get_data(G_OBJECT(list), "priv");
-  g_assert(priv);
-
-  priv->sel.func = func;
-  priv->sel.data = data;
-}
-
-/* default selection function enables edit and remove if a row is being */
-/* selected */
-static gboolean
-list_selection_function(GtkTreeSelection *selection, GtkTreeModel *model,
-			GtkTreePath *path, gboolean path_currently_selected,
-			gpointer list) {
-  list_priv_t *priv = g_object_get_data(G_OBJECT(list), "priv");
-  g_assert(priv);
-
-  GtkTreeIter iter;
-
-  if(gtk_tree_model_get_iter(model, &iter, path)) {
-    g_assert(gtk_tree_path_get_depth(path) == 1);
-
-    /* this is now handled by the "changed" event handler */
-    //    list_button_enable(GTK_WIDGET(list), LIST_BUTTON_REMOVE, TRUE);
-    //    list_button_enable(GTK_WIDGET(list), LIST_BUTTON_EDIT, TRUE);
-  }
-
-  if(priv->sel.func)
-    return priv->sel.func(selection, model, path, path_currently_selected, 
-			  priv->sel.data);
-
-  return TRUE; /* allow selection state to change */
-}
-
 static void on_row_activated(GtkTreeView *treeview,
 			     GtkTreePath        *path,
 			     GtkTreeViewColumn  *col,
@@ -325,12 +270,7 @@ void list_set_static_buttons(GtkWidget *list, int flags,
   /* add the three default buttons, but keep the disabled for now */
   if(cb_new) {
     priv->button.widget[0] = 
-      gtk_button_new_with_label(_((flags&LIST_BTN_NEW)?"New":"Add"));
-#ifdef FREMANTLE_PANNABLE_AREA
-    if(flags & LIST_BTN_BIG)
-      hildon_gtk_widget_set_theme_size(priv->button.widget[0], 
-	       (HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH));
-#endif
+      button_new_with_label(_((flags&LIST_BTN_NEW)?"New":"Add"));
     gtk_table_attach_defaults(GTK_TABLE(priv->table), 
 			      priv->button.widget[0], 0, 1, 0, 1);
     gtk_signal_connect(GTK_OBJECT(priv->button.widget[0]), "clicked", 
@@ -343,12 +283,7 @@ void list_set_static_buttons(GtkWidget *list, int flags,
     priv->button.widget[1] = cmenu_append(list, _("Edit"), 
 			  GTK_SIGNAL_FUNC(cb_edit), data);
 #else
-    priv->button.widget[1] = gtk_button_new_with_label(_("Edit"));
-#ifdef FREMANTLE_PANNABLE_AREA
-    if(flags & LIST_BTN_BIG)
-      hildon_gtk_widget_set_theme_size(priv->button.widget[1], 
-	       (HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH));
-#endif
+    priv->button.widget[1] = button_new_with_label(_("Edit"));
     gtk_table_attach_defaults(GTK_TABLE(priv->table), 
 			      priv->button.widget[1], 1, 2, 0, 1);
     gtk_signal_connect(GTK_OBJECT(priv->button.widget[1]), "clicked", 
@@ -362,12 +297,7 @@ void list_set_static_buttons(GtkWidget *list, int flags,
     priv->button.widget[2] = cmenu_append(list, _("Remove"), 
 			  GTK_SIGNAL_FUNC(cb_remove), data);
 #else
-    priv->button.widget[2] = gtk_button_new_with_label(_("Remove"));
-#ifdef FREMANTLE_PANNABLE_AREA
-    if(flags & LIST_BTN_BIG)
-      hildon_gtk_widget_set_theme_size(priv->button.widget[2], 
-	       (HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH));
-#endif
+    priv->button.widget[2] = button_new_with_label(_("Remove"));
     gtk_table_attach_defaults(GTK_TABLE(priv->table), 
 			      priv->button.widget[2], 2, 3, 0, 1);
     gtk_signal_connect(GTK_OBJECT(priv->button.widget[2]), "clicked", 
@@ -428,11 +358,37 @@ static gint on_list_destroy(GtkWidget *list, gpointer data) {
 
 static void changed(GtkTreeSelection *treeselection, gpointer user_data) {
   GtkWidget *list = (GtkWidget*)user_data;
+  list_priv_t *priv = g_object_get_data(G_OBJECT(list), "priv");
 
   GtkTreeModel *model;
   GtkTreeIter iter;
   gboolean selected = list_get_selected(list, &model, &iter);
 
+  /* scroll to selected entry if exactly one is selected */
+  if(selected) {
+    /* check if the entry isn't already visible */
+    GtkTreePath *start = NULL, *end = NULL;
+    GtkTreePath *path = gtk_tree_model_get_path(model, &iter);
+
+    gtk_tree_view_get_visible_range(GTK_TREE_VIEW(priv->view), &start, &end);
+      
+    /* check if path is before start of visible area or behin end of it */
+    if((start && (gtk_tree_path_compare(path, start)) < 0) ||
+       (end && (gtk_tree_path_compare(path, end) > 0)))
+      gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(priv->view), 
+				   path, NULL, TRUE, 0.5, 0.5);
+
+    if(start) gtk_tree_path_free(start);
+    if(end)   gtk_tree_path_free(end);
+    gtk_tree_path_free(path);
+  }
+
+  /* the change event handler is overridden */
+  if(priv->change.func) { 
+    priv->change.func(treeselection, priv->change.data);
+    return;
+  }
+
   list_button_enable(GTK_WIDGET(list), LIST_BUTTON_REMOVE, selected);
   list_button_enable(GTK_WIDGET(list), LIST_BUTTON_EDIT, selected);
 }
@@ -468,9 +424,6 @@ GtkWidget *list_new(void)
   GtkTreeSelection *sel = 
     gtk_tree_view_get_selection(GTK_TREE_VIEW(priv->view));
 
-  gtk_tree_selection_set_select_function(sel, 
-  	 list_selection_function, vbox, NULL);
-
   g_signal_connect(G_OBJECT(sel), "changed", G_CALLBACK(changed), vbox);
 
 #ifndef FREMANTLE_PANNABLE_AREA
@@ -505,3 +458,11 @@ GtkWidget *list_new(void)
   return vbox;
 }
 
+void list_override_changed_event(GtkWidget *list, 
+      void(*handler)(GtkTreeSelection*,gpointer), gpointer data) {
+  list_priv_t *priv = g_object_get_data(G_OBJECT(list), "priv");
+  g_assert(priv);
+
+  priv->change.func = handler;
+  priv->change.data = data;
+}
