@@ -24,6 +24,10 @@
 #include <curl/easy.h> /* new for v7 */
 #include <unistd.h>
 
+#ifdef FREMANTLE
+#include <hildon/hildon-text-view.h>
+#endif
+
 #define COLOR_ERR  "red"
 #define COLOR_OK   "darkgreen"
 
@@ -836,7 +840,7 @@ void osm_upload(appdata_t *appdata, osm_t *osm, project_t *project) {
 
 
   GtkWidget *dialog = 
-    misc_dialog_new(MISC_DIALOG_NOSIZE, _("Upload to OSM"),
+    misc_dialog_new(MISC_DIALOG_MEDIUM, _("Upload to OSM"),
 		    GTK_WINDOW(appdata->window),
 		    GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, 
 		    GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
@@ -867,34 +871,30 @@ void osm_upload(appdata_t *appdata, osm_t *osm, project_t *project) {
   table_attach_int(table, dirty.relations.dirty,   3, 4, 3, 4);
   table_attach_int(table, dirty.relations.deleted, 4, 5, 3, 4);
 
-  gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), table);
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), table, FALSE, FALSE, 0);
 
   /* ------------------------------------------------------ */
 
-  gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), 
-			      gtk_hseparator_new());
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), 
+		     gtk_hseparator_new(), FALSE, FALSE, 0);
 
   /* ------- add username and password entries ------------ */
 
   table = gtk_table_new(2, 2, FALSE);
   table_attach_label_l(table, _("Username:"), 0, 1, 0, 1);
-  GtkWidget *uentry = gtk_entry_new();
+  GtkWidget *uentry = entry_new();
   HILDON_ENTRY_NO_AUTOCAP(uentry);
   gtk_entry_set_text(GTK_ENTRY(uentry), appdata->settings->username);
   gtk_table_attach_defaults(GTK_TABLE(table),  uentry, 1, 2, 0, 1);
   table_attach_label_l(table, _("Password:"), 0, 1, 1, 2);
-  GtkWidget *pentry = gtk_entry_new();
+  GtkWidget *pentry = entry_new();
   HILDON_ENTRY_NO_AUTOCAP(pentry);
   gtk_entry_set_text(GTK_ENTRY(pentry), appdata->settings->password);
   gtk_entry_set_visibility(GTK_ENTRY(pentry), FALSE);
   gtk_table_attach_defaults(GTK_TABLE(table),  pentry, 1, 2, 1, 2);
-  gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), table);
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), table, FALSE, FALSE, 0);
 
-  GtkWidget *scrolled_win = gtk_scrolled_window_new(NULL, NULL);
-  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_win), 
-  				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled_win),
-				      GTK_SHADOW_IN);
+  GtkWidget *scrolled_win = misc_scrolled_window_new(TRUE);
 
   GtkTextBuffer *buffer = gtk_text_buffer_new(NULL);
   gtk_text_buffer_set_text(buffer, _("Please add a comment"), -1);
@@ -906,7 +906,13 @@ void osm_upload(appdata_t *appdata, osm_t *osm, project_t *project) {
   g_signal_connect(G_OBJECT(buffer), "changed",
 		   G_CALLBACK(callback_buffer_modified), dialog);
 
+#ifndef FREMANTLE
   GtkWidget *view = gtk_text_view_new_with_buffer(buffer);
+#else
+  GtkWidget *view = hildon_text_view_new();
+  hildon_text_view_set_buffer(HILDON_TEXT_VIEW(view), buffer);
+#endif
+
   gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(view), GTK_WRAP_WORD);
   gtk_text_view_set_editable(GTK_TEXT_VIEW(view), TRUE);
   gtk_text_view_set_left_margin(GTK_TEXT_VIEW(view), 2 );
@@ -970,7 +976,7 @@ void osm_upload(appdata_t *appdata, osm_t *osm, project_t *project) {
   gtk_dialog_set_response_sensitive(GTK_DIALOG(context->dialog), 
 				    GTK_RESPONSE_CLOSE, FALSE);
 
-  /* ------- main ui elelent is this text view --------------- */
+  /* ------- main ui element is this text view --------------- */
 
   GtkWidget *scrolled_window = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), 
