@@ -558,22 +558,19 @@ static void wms_server_selected(wms_server_context_t *context,
   }
 }
 
-static gboolean
-wms_server_selection_func(GtkTreeSelection *selection, GtkTreeModel *model,
-		     GtkTreePath *path, gboolean path_currently_selected,
-		     gpointer userdata) {
+static void
+wms_server_changed(GtkTreeSelection *selection, gpointer userdata) {
   wms_server_context_t *context = (wms_server_context_t*)userdata;
+
+  GtkTreeModel *model = NULL;
   GtkTreeIter iter;
-    
-  if(gtk_tree_model_get_iter(model, &iter, path)) {
+  
+  if(gtk_tree_selection_get_selected(selection, &model, &iter)) {
     wms_server_t *wms_server = NULL;
 
-    g_assert(gtk_tree_path_get_depth(path) == 1);
     gtk_tree_model_get(model, &iter, WMS_SERVER_COL_DATA, &wms_server, -1);
     wms_server_selected(context, wms_server);
   }
-  
-  return TRUE; /* allow selection state to change */
 }
 
 static void on_server_remove(GtkWidget *but, wms_server_context_t *context) {
@@ -648,7 +645,7 @@ gboolean wms_server_edit(wms_server_context_t *context, gboolean edit_name,
 		   GTK_FILL, 0, 0, 0);
   gtk_misc_set_alignment(GTK_MISC(label), 0.f, 0.5f);
   gtk_table_attach_defaults(GTK_TABLE(table), 
-		    name = gtk_entry_new(), 1, 2, 0, 1);
+		    name = entry_new(), 1, 2, 0, 1);
   gtk_entry_set_activates_default(GTK_ENTRY(name), TRUE);
   HILDON_ENTRY_NO_AUTOCAP(name);
   gtk_widget_set_sensitive(GTK_WIDGET(name), edit_name);
@@ -660,7 +657,7 @@ gboolean wms_server_edit(wms_server_context_t *context, gboolean edit_name,
 		   GTK_FILL, 0, 0, 0);
   gtk_misc_set_alignment(GTK_MISC(label), 0.f, 0.5f);
   gtk_table_attach_defaults(GTK_TABLE(table), 
-		    server = gtk_entry_new(), 1, 2, 1, 2);
+		    server = entry_new(), 1, 2, 1, 2);
   gtk_entry_set_activates_default(GTK_ENTRY(server), TRUE);
   HILDON_ENTRY_NO_AUTOCAP(server);
 
@@ -669,7 +666,7 @@ gboolean wms_server_edit(wms_server_context_t *context, gboolean edit_name,
 		   GTK_FILL, 0, 0, 0);
   gtk_misc_set_alignment(GTK_MISC(label), 0.f, 0.5f);
   gtk_table_attach_defaults(GTK_TABLE(table), 
-		    path = gtk_entry_new(), 1, 2, 2, 3);
+		    path = entry_new(), 1, 2, 2, 3);
   gtk_entry_set_activates_default(GTK_ENTRY(path), TRUE);
   HILDON_ENTRY_NO_AUTOCAP(path);
 
@@ -777,8 +774,7 @@ static GtkWidget *wms_server_widget(wms_server_context_t *context) {
 
   context->list = list_new(LIST_HILDON_WITHOUT_HEADERS);
 
-  list_set_selection_function(context->list, wms_server_selection_func, 
-			      context);
+  list_override_changed_event(context->list, wms_server_changed, context);
 
   list_set_columns(context->list, 
 		   _("Name"), WMS_SERVER_COL_NAME, LIST_FLAG_ELLIPSIZE, 
@@ -805,7 +801,7 @@ static GtkWidget *wms_server_widget(wms_server_context_t *context) {
   
   g_object_unref(context->store);
 
-  list_set_static_buttons(context->list, LIST_BTN_BIG, 
+  list_set_static_buttons(context->list, 0, 
 	  G_CALLBACK(on_server_add), G_CALLBACK(on_server_edit), 
 	  G_CALLBACK(on_server_remove), context);
 
@@ -890,75 +886,95 @@ static gboolean wms_server_dialog(appdata_t *appdata, wms_t *wms) {
 
 enum {
   LAYER_COL_TITLE = 0,
-  LAYER_COL_SELECTED,
   LAYER_COL_FITS,
   LAYER_COL_DATA,
   LAYER_NUM_COLS
 };
 
-static void
-layer_toggled(GtkCellRendererToggle *cell, const gchar *path_str,
-	      GtkListStore *store) {
-  GtkTreePath *path;
+#ifndef FREMANTLE
+/* we handle these events on our own in order to implement */
+/* a very direct selection mechanism (multiple selections usually */
+/* require the control key to be pressed). This interferes with */
+/* fremantle finger scrolling, but fortunately the fremantle */
+/* default behaviour already is what we want. */
+static gboolean on_view_clicked(GtkWidget *widget, GdkEventButton *event,
+				gpointer user_data) {
+  if(event->window == gtk_tree_view_get_bin_window(GTK_TREE_VIEW(widget))) {  
+    GtkTreePath *path;
+    
+    if(gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(widget), 
+		     event->x, event->y, &path, NULL, NULL, NULL)) {
+      GtkTreeSelection *sel = 
+	gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
+
+      if(!gtk_tree_selection_path_is_selected(sel, path))
+	gtk_tree_selection_select_path(sel, path);
+      else
+	gtk_tree_selection_unselect_path(sel, path);
+    }
+    return TRUE;
+  }
+  return FALSE;
+}
+#endif
+
+static void changed(GtkTreeSelection *sel, gpointer user_data) {
+  /* we need to know what changed in order to let the user acknowlege it! */
+
+  /* get view from selection ... */
+  GtkTreeView *view = gtk_tree_selection_get_tree_view(sel);
+  g_assert(view);
+
+  /* ... and get model from view */
+  GtkTreeModel *model = gtk_tree_view_get_model(view);
+  g_assert(model);
+
+  /* walk the entire store */
   GtkTreeIter iter;
+  gboolean one = FALSE, ok = gtk_tree_model_get_iter_first(model, &iter);
+  while(ok) {
+    wms_layer_t *layer = NULL;
 
-  path = gtk_tree_path_new_from_string(path_str);
-  gtk_tree_model_get_iter(GTK_TREE_MODEL(store), &iter, path);
+    gtk_tree_model_get(model, &iter, LAYER_COL_DATA, &layer, -1);    
+    g_assert(layer);
 
-  /* get current enabled flag */
-  gboolean enabled;
-  gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 
-		     LAYER_COL_SELECTED, &enabled, -1);
+    layer->selected = gtk_tree_selection_iter_is_selected(sel, &iter);
+    if(layer->selected) one = TRUE;
 
-  /* change it and store it */
-  enabled = !enabled;
-  gtk_list_store_set(store, &iter, LAYER_COL_SELECTED, enabled, -1);
-
-  /* and store it in the layer itself */
-  wms_layer_t *layer = NULL;
-  gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, LAYER_COL_DATA, &layer, -1);
-  layer->selected = enabled;
-
-  /* walk the entire store to get all values */
-  if(gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter)) {
-    gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 
-		       LAYER_COL_SELECTED, &enabled, -1);
-
-    while(gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter) &&
-	  !enabled) 
-      gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 
-			 LAYER_COL_SELECTED, &enabled, -1);
+    ok = gtk_tree_model_iter_next(model, &iter);
   }
 
-  GtkWidget *dialog = g_object_get_data(G_OBJECT(store), "dialog");
+  GtkWidget *dialog = gtk_widget_get_toplevel(GTK_WIDGET(view));
   gtk_dialog_set_response_sensitive(GTK_DIALOG(dialog), 
-				    GTK_RESPONSE_ACCEPT, enabled);
-  
-  gtk_tree_path_free(path);
+				    GTK_RESPONSE_ACCEPT, one);
 }
 
 static GtkWidget *wms_layer_widget(appdata_t *appdata, wms_layer_t *layer, 
 				   GtkWidget *dialog) {
+
+#ifndef FREMANTLE_PANNABLE_AREA
   GtkWidget *view = gtk_tree_view_new();
+#else
+  GtkWidget *view = hildon_gtk_tree_view_new(HILDON_UI_MODE_EDIT);
+#endif
+
+  /* change list mode to "multiple" */
+  GtkTreeSelection *selection = 
+    gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
+  gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
+
+#ifndef FREMANTLE
+  /* catch views button-press event for our custom handling */
+  g_signal_connect(view, "button-press-event", 
+		   G_CALLBACK(on_view_clicked), NULL); 
+#endif
 
   /* build the store */
   GtkListStore *store = gtk_list_store_new(LAYER_NUM_COLS, 
-      G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_POINTER);
-
-  g_object_set_data(G_OBJECT(store), "dialog", dialog);
-
-  /* --- "selected" column --- */
-  GtkCellRenderer *renderer = gtk_cell_renderer_toggle_new();
-  g_signal_connect(renderer, "toggled", G_CALLBACK(layer_toggled), store);
-  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),
-	-1, _(""), renderer, 
-        "active", LAYER_COL_SELECTED, 
-	"sensitive", LAYER_COL_FITS, 
-        "activatable", LAYER_COL_FITS, 
-	NULL);
+      G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_POINTER);
 
   /* --- "Title" column --- */
-  renderer = gtk_cell_renderer_text_new();
+  GtkCellRenderer *renderer = gtk_cell_renderer_text_new();
   g_object_set(renderer, "ellipsize", PANGO_ELLIPSIZE_END, NULL );
   GtkTreeViewColumn *column = gtk_tree_view_column_new_with_attributes(
 		 _("Title"), renderer, 
@@ -978,7 +994,6 @@ static GtkWidget *wms_layer_widget(appdata_t *appdata, wms_layer_t *layer,
     /* Append a row and fill in some data */
     gtk_list_store_append(store, &iter);
     gtk_list_store_set(store, &iter,
-	       LAYER_COL_SELECTED, FALSE,
 	       LAYER_COL_TITLE, layer->title,
 	       LAYER_COL_FITS, fits,
 	       LAYER_COL_DATA, layer,
@@ -988,6 +1003,9 @@ static GtkWidget *wms_layer_widget(appdata_t *appdata, wms_layer_t *layer,
   
   g_object_unref(store);
 
+  g_signal_connect(G_OBJECT(selection), "changed", G_CALLBACK(changed), layer);
+
+#ifndef FREMANTLE_PANNABLE_AREA
   /* put it into a scrolled window */
   GtkWidget *scrolled_window = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), 
@@ -995,8 +1013,13 @@ static GtkWidget *wms_layer_widget(appdata_t *appdata, wms_layer_t *layer,
   gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled_window), 
 				      GTK_SHADOW_ETCHED_IN);
   gtk_container_add(GTK_CONTAINER(scrolled_window), view);
-
   return scrolled_window;
+#else
+  /* put view into a pannable area */
+  GtkWidget *pannable_area = hildon_pannable_area_new();
+  gtk_container_add(GTK_CONTAINER(pannable_area), view);
+  return pannable_area;
+#endif
 }
 
 
@@ -1262,13 +1285,6 @@ void wms_import(appdata_t *appdata) {
   /* build complete url */
   old = url;
 
-#if 0
-  url = g_strdup_printf("%s&SRS=EPSG:4326&BBOX=%s,%s,%s,%s"
-			"&WIDTH=%d&HEIGHT=%d&FORMAT=%s"
-			"&reaspect=false", url,
-			minlon, minlat, maxlon, maxlat, wms->width, 
-			wms->height, formats[format]);
-#endif
   url = g_strdup_printf("%s&SRS=%s&BBOX=%s,%s,%s,%s"
 			"&WIDTH=%d&HEIGHT=%d&FORMAT=%s"
 			"&reaspect=false", url, srs,
@@ -1296,6 +1312,12 @@ void wms_import(appdata_t *appdata) {
   /* there should be a matching file on disk now */
   map_set_bg_image(appdata->map, filename);
 
+  gint x = appdata->osm->bounds->min.x + appdata->map->bg.offset.x;
+  gint y = appdata->osm->bounds->min.y + appdata->map->bg.offset.y;
+  canvas_image_move(appdata->map->bg.item, x, y, 
+		    appdata->map->bg.scale.x, appdata->map->bg.scale.y);
+
+
   g_free(filename);
   g_free(url);
 
@@ -1339,10 +1361,24 @@ void wms_load(appdata_t *appdata) {
   }
 }
 
-void wms_remove(appdata_t *appdata) {
+void wms_remove_file(project_t *project) {
   const char *exts[] = { "png", "gif", "jpg", "" };
   int i=0;
 
+  while(exts[i][0]) {
+    char *filename = 
+      g_strdup_printf("%s/wms.%s", project->path, exts[i]);
+
+    if(g_file_test(filename, G_FILE_TEST_EXISTS)) 
+      g_remove(filename);
+
+    g_free(filename);
+    i++;
+  }
+}
+
+void wms_remove(appdata_t *appdata) {
+
   /* this cancels any wms adjustment in progress */
   if(appdata->map->action.type == MAP_ACTION_BG_ADJUST)
     map_action_cancel(appdata);
@@ -1352,16 +1388,7 @@ void wms_remove(appdata_t *appdata) {
 
   map_remove_bg_image(appdata->map);
 
-  while(exts[i][0]) {
-    char *filename = g_strdup_printf("%s/wms.%s", appdata->project->path, 
-				     exts[i]);
-
-    if(g_file_test(filename, G_FILE_TEST_EXISTS)) 
-      g_remove(filename);
-
-    g_free(filename);
-    i++;
-  }
+  wms_remove_file(appdata->project);
 }
 
 struct server_preset_s {
